require "asciidoctor"
require "metanorma-iso"
require_relative "./front"
require_relative "./cleanup"
require_relative "./validate"

module Asciidoctor
  module BSI
    class Converter < ISO::Converter
      XML_ROOT_TAG = "bsi-standard".freeze
      XML_NAMESPACE = "https://www.metanorma.org/ns/bsi".freeze

      register_for "bsi"

      def boilerplate_file(_x_orig)
        File.join(@libdir, "bsi_intro_en.xml")
      end

      # TODO: we will not implement sections for now (Rules for Structure 6.4)
      #
      # TODO: we will not implement access date for now (Rules for Structure 10.3)

      def note(node)
        return commentary(node) if node.attr("type") == "commentary"

        super
      end

      def commentary_attrs(node)
        b = node.attr("beforeclauses") == "true" ? "true" : nil
        attr_code(keep_attrs(node).merge(
                    id: Metanorma::Utils::anchor_or_uuid(node),
                    type: "commentary",
                    target: node.attr("target"),
                    beforeclauses: b,
                  ))
      end

      def commentary(node)
        noko do |xml|
          xml.admonition **commentary_attrs(node) do |a|
            node.title.nil? or a.name { |name| name << node.title }
            wrap_in_para(node, a)
          end
        end.join("\n")
      end

      def html_converter(node)
        if node.nil?
          IsoDoc::BSI::HtmlConvert.new({})
        else
          IsoDoc::BSI::HtmlConvert.new(html_extract_attributes(node))
        end
      end

      def pdf_converter(node)
        return if node.attr("no-pdf")

        if node.nil?
          IsoDoc::BSI::PdfConvert.new({})
        else
          IsoDoc::BSI::PdfConvert.new(doc_extract_attributes(node))
        end
      end

      def presentation_xml_converter(node)
        if node.nil?
          IsoDoc::BSI::PresentationXMLConvert.new({})
        else
          IsoDoc::BSI::PresentationXMLConvert.new(doc_extract_attributes(node))
        end
      end

      def sts_converter(node)
        return if node.attr("no-pdf")

        if node.nil?
          IsoDoc::BSI::StsConvert.new({})
        else
          IsoDoc::BSI::StsConvert.new(html_extract_attributes(node))
        end
      end
    end
  end
end
