module Asciidoctor
  module BSI
    class Converter < ISO::Converter
      def org_abbrev
        super.merge("The British Standards Institution" => "BSI")
      end

      def metadata_author(node, xml)
        publishers = node.attr("publisher") || "BSI"
        csv_split(publishers)&.each do |p|
          xml.contributor do |c|
            c.role **{ type: "author" }
            c.organization do |a|
              organization(a, p, false, node, !node.attr("publisher"))
            end
          end
        end
      end

      def metadata_publisher(node, xml)
        publishers = node.attr("publisher") || "BSI"
        csv_split(publishers)&.each do |p|
          xml.contributor do |c|
            c.role **{ type: "publisher" }
            c.organization do |a|
              organization(a, p, true, node, !node.attr("publisher"))
            end
          end
        end
      end

      def metadata_copyright(node, xml)
        pub = node.attr("copyright-holder") || node.attr("publisher") || "BSI"
        csv_split(pub)&.each do |p|
          xml.copyright do |c|
            c.from (node.attr("copyright-year") || Date.today.year)
            c.owner do |owner|
              owner.organization do |o|
                organization(o, p, true, node, !node.attr("copyright-holder") ||
                             node.attr("publisher"))
              end
            end
          end
        end
      end

      def iso_id(node, xml)
        return unless node.attr("docnumber")

        part = node&.attr("partnumber")
        dn = add_id_parts(node.attr("docnumber"), part, nil)
        dn = id_stage_prefix(dn, node, false)
        xml.docidentifier dn, **attr_code(type: "BS")
      end

      def metadata_doctype(node, xml)
        xml.doctype doctype(node)
      end

      def relaton_relations
        super + %w(obsoletes adopted-from bsi-related)
      end

      def get_typeabbr(node, amd = false)
        case doctype(node)
        when "publicly-available-specification" then "PAS "
        when "british-standard" then "BS "
        when "draft-for-development" then "DD "
        when "privately-subscribed-standard" then "PSS "
        when "flex-standard" then "Flex "
        else
          super
        end
      end

      def id_stage_abbr(stage, substage, node, bare = false)
        ret = super
        if %w(publicly-available-specification).include?(doctype(node)) &&
            !%w(D FD).include?(ret)
          ret = "#{ret} "
        end
        ret
      end

      def id_stage_prefix(docnum, node, force_year)
        stage = get_stage(node)
        typeabbr = get_typeabbr(node)
        typeabbr.nil? || typeabbr.empty? and typeabbr = "BS "
        docnum = if stage && (stage.to_i < 60)
                   unpub_stage_prefix(docnum, stage, typeabbr, node)
                 else "#{typeabbr}#{docnum}"
                 end
        (force_year || !(stage && (stage.to_i < 60))) and
          docnum = id_add_year(docnum, node)
        docnum
      end

      def unpub_stage_prefix(docnum, stage, typeabbr, node)
        typeabbr = "" if typeabbr == "BS "
        super
      end

      # TODO: not yet implemented prefixes for subsectors explicitly, Rules for Structure Annex F
    end
  end
end
