module Asciidoctor
  module BSI
    class Converter < ISO::Converter
      # Rules for Structure 23.3
      def listcount_validate(doc)
        return if @novalid

        doc.xpath("//clause[not(.//clause)] | //annex").each do |c|
          next if c.xpath(".//ol").empty?

          ols = c.xpath(".//ol") - c.xpath(".//ul//ol | .//ol//ol")
          ols.size > 3 and
            style_warning(c, "More than 3 ordered lists in a numbered clause",
                          nil)
        end
      end

      # Rules for Structure 23.5
      def list_punctuation(doc)
        return if @novalid

        ((doc.xpath("//ol") - doc.xpath("//ul//ol | //ol//ol")) +
         (doc.xpath("//ul") - doc.xpath("//ul//ul | //ol//ul"))).each do |list|
          prec = list.previous_element
          prec&.name == "p" or
            style_warning(list, "All lists must be preceded by "\
                          "introductory phrase", nil)
          list_punctuation1(list, prec&.text)
        end
      end

      def list_punctuation1(list, prectext)
        entries = list.xpath(".//li")
        case prectext&.sub(/^.*?(\S)\s*$/, "\\1")
        when ":"
          list.xpath(".//li").each_with_index do |li, i|
            list_semicolon_phrase(li, i == entries.size - 1)
          end
        when "." then entries.each { |li| list_full_sentence(li) }
        else style_warning(list, "All lists must be preceded by "\
                          "colon or full stop", prectext)
        end
      end

      def list_semicolon_phrase(elem, last)
        text = elem.text.strip
        text.match?(/^[^A-Za-z]*[a-z]/) or
          style_warning(elem, "List entry after colon must start with "\
                        "lowercase letter", text)
        punct = text.sub(/^.*?(\S)\s*$/, "\\1")
        if last
          punct == "." or style_warning(elem, "Final list entry after colon "\
                                        "must end with full stop", text)
        else
          punct == ";" or style_warning(elem, "List entry after colon must "\
                                        "end with semicolon", text)
        end
      end

      def list_full_sentence(elem)
        text = elem.text.strip
        text.match?(/^[^A-Za-z]*[A-Z]/) or
          style_warning(elem, "List entry after full stop must start with "\
                        "uppercase letter", text)
        punct = text.sub(/^.*?(\S)\s*$/, "\\1")
        punct == "." or
          style_warning(elem, "List entry after full stop must "\
                        "end with full stop", text)
      end
    end
  end
end
