require "isodoc"
require "metanorma-iso"

module IsoDoc
  module BSI
    module BaseConvert
      def boilerplate(node, out)
        # processed in foreword instead
      end

      def foreword(isoxml, out)
        f = isoxml.at(ns("//foreword"))
        b = isoxml.at(ns("//boilerplate/legal-statement"))
        page_break(out)
        bsi_orgname(out)
        middle_title(isoxml, out)
        out.div **attr_code(id: f ? f["id"] : "") do |s|
          s.h1(**{ class: "ForewordTitle" }) { |h1| h1 << @i18n.foreword }
          @meta.get[:doctype] == "Amendment" or
            s.div **attr_code(class: "boilerplate_legal") do |s1|
              b&.elements&.each { |e| parse(e, s1) }
            end
          f&.elements&.each { |e| parse(e, s) unless e.name == "title" }
        end
      end

      def bsi_orgname(out)
        out.p(**{ class: "zzSTDTitle1" }) { |p| p << @i18n.get["BSI"] }
        out.p(**{ class: "zzSTDTitle1" }) { |p| p << "____________" }
        out.p(**{ class: "zzSTDTitle1" }) { |p| p << "&nbsp;" }
      end

      def middle_title(_isoxml, out)
        title1 = @meta.get[:doctitlemain]&.sub(/\s+$/, "")
        @meta.get[:doctitleintro] and
          title1 = "#{@meta.get[:doctitleintro]} &mdash; #{title1}"
        if @meta.get[:doctitlepart]
          title1 += " &mdash;"
          title2 = @meta.get[:doctitlepart]&.sub(/\s+$/, "")
          @meta.get[:doctitlepartlabel] and
            title2 = "#{@meta.get[:doctitlepartlabel]}: #{title2}"
        end
        out.p(**{ class: "zzSTDTitle1" }) do |p|
          p.b { |b| b << title1 }
        end
        if @meta.get[:doctitlepart]
          out.p(**{ class: "zzSTDTitle1" }) { |p| p << "&nbsp;" }
          out.p(**{ class: "zzSTDTitle2" }) do |p|
            p.b { |b| b << title2 }
          end
        end
        out.p(**{ class: "zzSTDTitle1" }) { |p| p << "&nbsp;" }
      end

      def deprecated_term_parse(node, out)
        out.p **{ class: "DeprecatedTerms", style: "text-align:left;" } do |p|
          p << l10n("#{@i18n.deprecated}: ")
          node.children.each { |c| parse(c, p) }
        end
      end

      def admitted_term_parse(node, out)
        out.p **{ class: "AltTerms", style: "text-align:left;" } do |p|
          node.children.each { |c| parse(c, p) }
        end
      end

      def admonition_parse(node, div)
        return commentary_parse(node, div) if node["type"] == "commentary"

        super
      end

      def admonition_name_parse(_node, div, name)
        div.b do |b|
          name.children.each { |n| parse(n, b) }
        end
        div << ". "
      end

      def commentary_parse(node, div)
        name = node&.at(ns("./name"))&.remove
        div.p do |p|
          name and p.span **{ class: "note_label" } do |s|
            name.children.each { |n| parse(n, s) }
          end
        end
        node.element_children.each { |n| parse(n, div) }
      end
    end
  end
end
