require "metanorma/processor"

module Metanorma
  module BSI
    class Processor < Metanorma::Processor

      def initialize # rubocop:disable Lint/MissingSuper
        @short = :bsi
        @input_format = :asciidoc
        @asciidoctor_backend = :bsi
      end

      def output_formats
        super.merge(
          html: "html",
          pdf: "pdf",
          sts: "sts.xml",
        )
      end

      def fonts_manifest
        {
          "Tahoma" => nil,
          "STIX Two Math" => nil,
          "Source Han Sans" => nil,
          "Source Han Sans Normal" => nil,
          "Courier New" => nil,
        }
      end

      def version
        "Metanorma::BSI #{Metanorma::BSI::VERSION}"
      end

      def output(xml, inname, outname, format, options = {})
        case format
        when :html
          IsoDoc::BSI::HtmlConvert.new(options).convert(inname, xml, nil, outname)
        when :pdf
          IsoDoc::BSI::PdfConvert.new(options).convert(inname, xml, nil, outname)
        when :sts
          IsoDoc::BSI::StsConvert.new(options).convert(inname, xml, nil, outname)
        when :presentation
          IsoDoc::BSI::PresentationXMLConvert.new(options).convert(inname, xml, nil, outname)
        else
          super
        end
      end
    end
  end
end
